@@ -44,9 +44,11 @@
 #include  <QWidget>
 #include  <QUrl>
 #include  <QFileInfo>
+#include  <QNetworkRequest>
 #include  <QDir>
 #include  <QSettings>
 #include  <QProcess>
+#include  <QThread>
 
 #include  "Helpers.h"
 #include  "Widgets.h"
@@ -54,6 +56,7 @@
 #include  "OsFile.h"
 
 #include  "DataStream.h" // for translation
+#include  "MpegFrame.h"
 
 
 using namespace std;
@@ -345,269 +348,108 @@ void logToGlobalFile(const string& s) //tttc make sure it is disabled in public
 
 
 
+namespace {
+
+struct Decoder {
+    QString decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid);
+    string decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid);
+};
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-#define DECODE_CHECK(COND, MSG) { if (!(COND)) { bRes = false; return MSG; } }
-
-namespace
+/**
+ * @param bfr MPEG header candidate; must have at least 4 bytes
+ * @param pbIsValid optional; if present,
+ * @return error, as a translated string, if bfr is invalid; empty string if buffer is valid
+ */
+QString Decoder::decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid)
 {
-
-    struct Decoder
-    {
-        enum Version { MPEG1, MPEG2 };
-        enum Layer { LAYER1, LAYER2, LAYER3 };
-        enum ChannelMode { STEREO, JOINT_STEREO, DUAL_CHANNEL, SINGLE_CHANNEL };
-
-        Version m_eVersion;
-        Layer m_eLayer;
-        int m_nBitrate;
-        int m_nFrequency;
-        int m_nPadding;
-        ChannelMode m_eChannelMode;
-
-        int m_nSize;
-        bool m_bCrc;
-
-        const char* getSzVersion() const;
-        const char* getSzLayer() const;
-        const char* getSzChannelMode() const;
-
-        QString initialize(const unsigned char* bfr, bool* pbIsValid);
-        QString decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid);
-        string decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid);
-    };
-
-    const char* Decoder::getSzVersion() const
-    {
-        static const char* s_versionName[] = { QT_TRANSLATE_NOOP("DataStream", "MPEG-1"), QT_TRANSLATE_NOOP("DataStream", "MPEG-2") };
-        return s_versionName[m_eVersion];
-    }
-
-    const char* Decoder::getSzLayer() const
-    {
-        static const char* s_layerName[] = { QT_TRANSLATE_NOOP("DataStream", "Layer I"), QT_TRANSLATE_NOOP("DataStream", "Layer II"), QT_TRANSLATE_NOOP("DataStream", "Layer III") };
-        return s_layerName[m_eLayer];
-    }
-
-    const char* Decoder::getSzChannelMode() const
-    {
-        static const char* s_channelModeName[] = { QT_TRANSLATE_NOOP("DataStream", "Stereo"), QT_TRANSLATE_NOOP("DataStream", "Joint stereo"), QT_TRANSLATE_NOOP("DataStream", "Dual channel"), QT_TRANSLATE_NOOP("DataStream", "Single channel") };
-        return s_channelModeName[m_eChannelMode];
-    }
-
-
-    QString Decoder::initialize(const unsigned char* bfr, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor; note that they also share translations
-    {
-        bool b;
-        bool& bRes (0 == pbIsValid ? b : *pbIsValid);
-        bRes = true;
-        const unsigned char* pHeader (bfr);
-    //inspect(bfr, BFR_SIZE);
-        DECODE_CHECK (0xff == *pHeader && 0xe0 == (0xe0 & *(pHeader + 1)), DataStream::tr("Not an MPEG frame. Synch missing."));
-        ++pHeader;
-
-        {
-            int nVer ((*pHeader & 0x18) >> 3);
-            switch (nVer)
-            {//TRACE
-            case 0x00: bRes = false; return DataStream::tr("Not an MPEG frame. Unsupported version (2.5)."); //ttt2 see about supporting this: search for MPEG1 to find other places
-                // ttt2 in a way it would make more sense to warn that it's not supported, with "MP3_THROW(SUPPORT, ...)", but before warn, make sure it's a valid 2.5 frame, followed by another frame ...
-
-            case 0x02: m_eVersion = MPEG2; break;
-            case 0x03: m_eVersion = MPEG1; break;
-
-            default: bRes = false; return DataStream::tr("Not an MPEG frame. Invalid version.");
-            }
-        }
-
-        {
-            int nLayer ((*pHeader & 0x06) >> 1);
-            switch (nLayer)
-            {
-            case 0x01: m_eLayer = LAYER3; break;
-            case 0x02: m_eLayer = LAYER2; break;
-            case 0x03: m_eLayer = LAYER1; break;
-
-            default: bRes = false; return DataStream::tr("Not an MPEG frame. Invalid layer.");
-            }
-        }
-
-        {
-            m_bCrc = !(*pHeader & 0x01);
-        }
-
-        ++pHeader;
-        {
-            static int s_bitrates [14][5] =
-                {
-                    {  32,      32,      32,      32,       8 },
-                    {  64,      48,      40,      48,      16 },
-                    {  96,      56,      48,      56,      24 },
-                    { 128,      64,      56,      64,      32 },
-                    { 160,      80,      64,      80,      40 },
-                    { 192,      96,      80,      96,      48 },
-                    { 224,     112,      96,     112,      56 },
-                    { 256,     128,     112,     128,      64 },
-                    { 288,     160,     128,     144,      80 },
-                    { 320,     192,     160,     160,      96 },
-                    { 352,     224,     192,     176,     112 },
-                    { 384,     256,     224,     192,     128 },
-                    { 416,     320,     256,     224,     144 },
-                    { 448,     384,     320,     256,     160 }
-                };
-            int nRateIndex ((*pHeader & 0xf0) >> 4);
-            DECODE_CHECK (nRateIndex >= 1 && nRateIndex <= 14, DataStream::tr("Not an MPEG frame. Invalid bitrate."));
-            int nTypeIndex (m_eVersion*3 + m_eLayer);
-            if (nTypeIndex == 5) { nTypeIndex = 4; }
-            m_nBitrate = s_bitrates[nRateIndex - 1][nTypeIndex]*1000;
-        }
-
-        {
-            int nSmpl ((*pHeader & 0x0c) >> 2);
-            switch (m_eVersion)
-            {
-            case MPEG1:
-                switch (nSmpl)
-                {
-                case 0x00: m_nFrequency = 44100; break;
-                case 0x01: m_nFrequency = 48000; break;
-                case 0x02: m_nFrequency = 32000; break;
-
-                default: bRes = false; return DataStream::tr("Not an MPEG frame. Invalid frequency for MPEG1.");
-                }
-                break;
-
-            case MPEG2:
-                switch (nSmpl)
-                {
-                case 0x00: m_nFrequency = 22050; break;
-                case 0x01: m_nFrequency = 24000; break;
-                case 0x02: m_nFrequency = 16000; break;
-
-                default: bRes = false; return DataStream::tr("Not an MPEG frame. Invalid frequency for MPEG2.");
-                }
-                break;
-
-            default: CB_THROW(CbRuntimeError); // it should have thrown before getting here
-            }
-        }
-
-        {
-            m_nPadding = (0x02 & *pHeader) >> 1;
-        }
-
-        ++pHeader;
-        {
-            int nChMode ((*pHeader & 0xc0) >> 6);
-            m_eChannelMode = (ChannelMode)nChMode;
-        }
-
-        switch (m_eLayer)
-        {
-        case LAYER1:
-            m_nSize = (12*m_nBitrate/m_nFrequency + m_nPadding)*4;
-            break;
-
-        case LAYER2:
-            m_nSize = 144*m_nBitrate/m_nFrequency + m_nPadding;
-            break;
-
-        case LAYER3:
-            m_nSize = (MPEG1 == m_eVersion ? 144*m_nBitrate/m_nFrequency + m_nPadding : 72*m_nBitrate/m_nFrequency + m_nPadding);
-            break;
-
-        default: CB_THROW(CbRuntimeError); // it should have thrown before getting here
-        }
-
-        return "";
-    }
-
-
-    QString Decoder::decodeMpegFrame(const unsigned char* bfr, const char* szSep, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
-    {
-        QString s (initialize(bfr, pbIsValid));
-        if (!s.isEmpty()) { return s; }
-
-        //ostringstream out;
-        /*out << getSzVersion() << " " << getSzLayer() << ", " << m_nBitrate/1000 << "kbps, " << m_nFrequency << "Hz, " << getSzChannelMode() << ", padding=" << (m_nPadding ? "true" : "false") << ", length " <<
-                m_nSize << " (0x" << hex << m_nSize << dec << ")";*/
-
-
-        /*out << boolalpha <<
-               getSzVersion() << " " <<
-               getSzLayer() <<
-               szSep <<
-               getSzChannelMode()/ *4* / <<
-                szSep <<
-               m_nFrequency << "Hz" <<
-               szSep <<
-               m_nBitrate / *8* / << "bps" <<
-               szSep << "CRC=" <<
-               boolAsYesNo(m_bCrc) <<
-               szSep / *11* /<< "length " <<
-               m_nSize <<
-               " (0x" << hex << m_nSize << dec << ")" <<
-               szSep / *14* /<< "padding=" <<
-               (m_nPadding ? "true" : "false");*/
-
-
-
-
+    NoteColl noteColl;
+    bool b;
+    bool& bRes (0 == pbIsValid ? b : *pbIsValid);
+    bRes = true;
+    try {
+        MpegFrameBase frame(noteColl, reinterpret_cast<const char*>(bfr));
         return DataStream::tr("%1 %2%3%4%5%6Hz%7%8bps%9CRC=%10%11length %12 (0x%13)%14padding=%15")
-                .arg(DataStream::tr(getSzVersion()))
-                .arg(DataStream::tr(getSzLayer()))
+                .arg(DataStream::tr(frame.getSzVersion()))
+                .arg(DataStream::tr(frame.getSzLayer()))
                 .arg(szSep)
-                .arg(DataStream::tr(getSzChannelMode()))
+                .arg(DataStream::tr(frame.getSzChannelMode()))
                 .arg(szSep)
-                .arg(m_nFrequency)
+                .arg(frame.getFrequency())
                 .arg(szSep)
-                .arg(m_nBitrate)
+                .arg(frame.getBitrate())
                 .arg(szSep)
-                .arg(GlobalTranslHlp::tr(boolAsYesNo(m_bCrc)))
+                .arg(GlobalTranslHlp::tr(boolAsYesNo(frame.getCrcUsage())))
                 .arg(szSep)
-                .arg(m_nSize)
-                .arg(m_nSize, 0, 16)
+                .arg(frame.getSize())
+                .arg(frame.getSize(), 0, 16)
                 .arg(szSep)
-                .arg(GlobalTranslHlp::tr(boolAsYesNo(m_nPadding)));
+                .arg(GlobalTranslHlp::tr(boolAsYesNo(frame.getPadding() != 0)));
+    } catch (const exception& ex) {
+        bRes = false;
+        const vector<Note*>& notes = noteColl.getList();
+        if (!notes.empty()) {
+            return notes[0]->getDescription();
+        }
+        return "err";
     }
 
+    //ostringstream out;
+    /*out << getSzVersion() << " " << getSzLayer() << ", " << m_nBitrate/1000 << "kbps, " << m_nFrequency << "Hz, " << getSzChannelMode() << ", padding=" << (m_nPadding ? "true" : "false") << ", length " <<
+            m_nSize << " (0x" << hex << m_nSize << dec << ")";*/
 
-    string Decoder::decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
-    {
-        QString s (initialize(bfr, pbIsValid)); // !!! XML is not translated
-        if (!s.isEmpty()) { return convStr(s); }
 
+    /*out << boolalpha <<
+           getSzVersion() << " " <<
+           getSzLayer() <<
+           szSep <<
+           getSzChannelMode()/ *4* / <<
+            szSep <<
+           m_nFrequency << "Hz" <<
+           szSep <<
+           m_nBitrate / *8* / << "bps" <<
+           szSep << "CRC=" <<
+           boolAsYesNo(m_bCrc) <<
+           szSep / *11* /<< "length " <<
+           m_nSize <<
+           " (0x" << hex << m_nSize << dec << ")" <<
+           szSep / *14* /<< "padding=" <<
+           (m_nPadding ? "true" : "false");*/
+}
+
+
+// Used only for TruncatedMpegDataStream
+string Decoder::decodeMpegFrameAsXml(const unsigned char* bfr, bool* pbIsValid) //ttt2 perhaps unify with MpegFrameBase::MpegFrameBase(), using the char* constructor
+{
+    NoteColl noteColl;
+    bool b;
+    bool& bRes (0 == pbIsValid ? b : *pbIsValid);
+    bRes = true;
+    try {
+        MpegFrameBase frame(noteColl, reinterpret_cast<const char*>(bfr));
         ostringstream out;
-        out << " version=\"" << getSzVersion() << "\""
-            << " layer=\"" << getSzLayer() << "\""
-            << " channelMode=\"" << getSzChannelMode() << "\""
-            << " frequency=\"" << m_nFrequency << "\""
-            << " bps=\"" << m_nBitrate << "\""
-            << " crc=\"" << boolAsYesNo(m_bCrc) << "\""
+        out << " version=\"" << frame.getSzVersion() << "\""
+            << " layer=\"" << frame.getSzLayer() << "\""
+            << " channelMode=\"" << frame.getSzChannelMode() << "\""
+            << " frequency=\"" << frame.getFrequency() << "\""
+            << " bps=\"" << frame.getBitrate() << "\""
+            << " crc=\"" << boolAsYesNo(frame.getCrcUsage()) << "\""
 
-            << " mpegSize=\"" << m_nSize << "\""
-            << " padding=\"" << boolAsYesNo(m_nPadding) << "\""; // !!! XML isn't translated
+            << " mpegSize=\"" << frame.getSize() << "\""
+            << " padding=\"" << boolAsYesNo(frame.getPadding() != 0) << "\""; // !!! XML isn't translated
 
         return out.str();
+    } catch (const exception& ex) {
+        bRes = false;
+        const vector<Note*>& notes = noteColl.getList();
+        if (!notes.empty()) {
+            return notes[0]->getDescription();
+        }
+        return "err";
     }
-} // namespace
+}
 
+}  // namespace
 
 
 string decodeMpegFrame(unsigned int x, const char* szSep, bool* pbIsValid /* = 0*/)
@@ -625,7 +467,9 @@ string decodeMpegFrame(const char* bfr, const char* szSep, bool* pbIsValid /* =
 {
     Decoder d;
     const unsigned char* q (reinterpret_cast<const unsigned char*>(bfr));
-    return convStr(d.decodeMpegFrame(q, szSep, pbIsValid));
+    //Decoder1().decodeMpegFrame(q, szSep, pbIsValid);
+    const string& res = convStr(d.decodeMpegFrame(q, szSep, pbIsValid));
+    return res;
 }
 
 
@@ -664,6 +508,19 @@ const string& getPathSepAsStr()
 }
 
 
+string join(const vector<string>& vec, const char* szSep)
+{
+    // https://iq.opengenus.org/convert-vector-to-string-in-cpp/
+    ostringstream stream;
+    if(!vec.empty())
+    {
+        copy(vec.begin(), vec.end() - 1, ostream_iterator<string>(stream, szSep));
+        stream << vec.back();
+    }
+    return stream.str();
+}
+
+
 
 
 streampos getSize(istream& in)
@@ -740,7 +597,7 @@ vector<QString> convStr(const vector<string>& v)
 namespace {
 
 struct DesktopDetector {
-    enum Desktop { Unknown, Gnome2 = 1, Gnome3 = 2, Kde3 = 4, Kde4 = 8, Gnome = Gnome2 | Gnome3, Kde = Kde3 | Kde4 };
+    enum Desktop { Unknown, Gnome2 = 1, Gnome3 = 2, Kde3 = 4, Kde4 = 8, Kde5 = 16, Gnome = Gnome2 | Gnome3, Kde = Kde3 | Kde4 | Kde5 };
     DesktopDetector();
     Desktop m_eDesktop;
     const char* m_szDesktop;
@@ -760,6 +617,7 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
 
     bool bIsKde (false);
     bool bIsKde4 (false);
+    bool bIsKde5 (false);
 
     while (fs)
     {
@@ -767,7 +625,7 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
         {
             string strCmdLineName (fs.getName());
 
-            if (isdigit(strCmdLineName[6]))
+            if (isdigit(strCmdLineName[6])) // strCmdLineName is "/proc/xyz"; usually xyz is a number, which is a process id, but there are also files, whose names start with letters
             {
 
 #if 0
@@ -808,7 +666,7 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
                 //szBfr[0] = 0;
 
 
-                strCmdLineName += "/cmdline";
+                strCmdLineName += "/cmdline"; // we usually get something like "/proc/156/cmdline", which is the command line of process 156
                 //cout << strCmdLineName << endl;
                 ifstream in (strCmdLineName.c_str());
                 if (in)
@@ -832,7 +690,12 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
                     // for i in `ls /proc | grep '^[0-9]'` ; do a=`cat /proc/$i/cmdline 2>/dev/null` ; echo $a | grep kde4/libexec ; done
                     if (string::npos != strBfr.find("kde4/libexec/start")) // ttt2 probably works only on Suse and only in some cases
                     {
-                        bIsKde4 = true; //ttt9 update
+                        bIsKde4 = true;
+                    }
+
+                    if (string::npos != strBfr.find("/kded5")) // ttt2 probably works only on Suse and only in some cases
+                    {
+                        bIsKde5 = true; //ttt9 update for KDE 6 (Jan 2024)
                     }
                 }//*/
 
@@ -845,7 +708,11 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
 
     if (m_eDesktop == Unknown)
     {
-        if (bIsKde4)
+        if (bIsKde5)
+        {
+            m_eDesktop = Kde5;
+        }
+        else if (bIsKde4)
         {
             m_eDesktop = Kde4;
         }
@@ -870,6 +737,7 @@ DesktopDetector::DesktopDetector() : m_eDesktop(Unknown)
     case Gnome3: m_szDesktop = "Gnome 3"; break;
     case Kde3: m_szDesktop = "KDE 3"; break;
     case Kde4: m_szDesktop = "KDE 4"; break;
+    case Kde5: m_szDesktop = "KDE 5"; break;
     default: m_szDesktop = "Unknown";
     }
     //cout << "desktop: " << m_eDesktop << endl;
@@ -915,7 +783,7 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 //ttt0 look at Qt::CustomizeWindowHint
 #ifndef WIN32
     //Qt::WindowFlags getMainWndFlags() { return isRunningOnGnome() ? Qt::Window : Qt::WindowTitleHint; } // !!! these are incorrect, but seem the best option; the values used for Windows are supposed to be OK; they work as expected with KDE but not with Gnome (asking for maximize button not only fails to show it, but causes the "Close" button to disappear as well); Since in KDE min/max buttons are shown when needed anyway, it's sort of OK // ttt2 see if there is workaround/fix
-    Qt::WindowFlags getMainWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint : Qt::Window; }
+    Qt::WindowFlags getMainWndFlags() { /*const DesktopDetector& dd = getDesktopDetector();*/ return /*dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowCloseButtonHint :*/ Qt::Window; }
 #if QT_VERSION >= 0x040500
     //Qt::WindowFlags getDialogWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint : (dd.onDesktop(DesktopDetector::Gnome3) ? Qt::Window : Qt::WindowTitleHint); }
     Qt::WindowFlags getDialogWndFlags() { const DesktopDetector& dd = getDesktopDetector(); return dd.onDesktop(DesktopDetector::Kde) ? Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint : (/*dd.onDesktop(DesktopDetector::Gnome3) ? Qt::Window :*/ Qt::WindowTitleHint); }
@@ -926,11 +794,11 @@ Ideally a modal dialog should minimize its parent. If that's not possible, it sh
 #endif
     Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; }
 #else
-    Qt::WindowFlags getMainWndFlags() { 
-		return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint;
-	} // minimize, maximize, no "what's this"
+    Qt::WindowFlags getMainWndFlags() {
+        return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint;
+    } // minimize, maximize, no "what's this"
     Qt::WindowFlags getDialogWndFlags() { return Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint; } // minimize, no "what's this"
-    Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint; } // no "what's this" // ttt2 review if we want Qt::WindowCloseButtonHint
+    Qt::WindowFlags getNoResizeWndFlags() { return Qt::WindowTitleHint | Qt::WindowCloseButtonHint; } // no "what's this"
 #endif
 
 
@@ -1156,7 +1024,7 @@ vector<QString> getLocalHelpDirs()
         //s_v.push_back("/home/ciobi/cpp/Mp3Utils/mp3diags/trunk/mp3diags/doc/html/");
         s_v.push_back(QString("/usr/share/") + getHelpPackageName() + "-doc/html/"); //ttt0 lower/uppercase variations
         s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "/html/");
-        s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "-1.5.01/html/");
+        s_v.push_back(QString("/usr/share/doc/") + getHelpPackageName() + "-1.5.03/html/");
 #else
         wchar_t wszModule [200];
         int nRes (GetModuleFileName(0, wszModule, 200));
@@ -1173,6 +1041,11 @@ vector<QString> getLocalHelpDirs()
     return s_v;
 }
 
+string getWebHelpUrl()
+{
+    return string("https://mp3diags.sourceforge.net") + getWebBranch() + "/";
+}
+
 
 // opens a web page from the documentation in the default browser;
 // first looks in several places on the local computer; if the file can't be found there, it goes to SourceForge
@@ -1192,7 +1065,7 @@ void openHelp(const string& strFileName)
     QString qs (strDir);
     if (qs.isEmpty())
     {
-        qs = "http://mp3diags.sourceforge.net" + QString(getWebBranch()) + "/";
+        qs = convStr(getWebHelpUrl());
     }
     else
     {
@@ -1738,6 +1611,21 @@ LastStepTracer::~LastStepTracer()
     traceLastStep(" < " + m_s, -1);
 }
 
+
+//=============================================================================================
+
+string getCurrentThreadInfo()
+{
+    //return convStr(QThread::currentThread()->objectName());
+    Qt::HANDLE pVoid = QThread::currentThreadId();
+    //return convStr(QString("%1").arg(1));
+    char a[20];
+    sprintf(a, "%p", pVoid);
+    return a;
+}
+
+
+
 //=============================================================================================
 //=============================================================================================
 //=============================================================================================
@@ -1848,6 +1736,8 @@ Timer::~Timer() {
     return a;
 }
 
+
+
 //=============================================================================================
 //=============================================================================================
 //=============================================================================================
@@ -1855,10 +1745,20 @@ Timer::~Timer() {
 void decreaseRowHeaderFont(QTableView& qtableView)
 {
 #ifdef WIN32
-	QFont font(qtableView.verticalHeader()->font());
-	auto sz(font.pointSizeF());
-	font.setPointSizeF(sz * 0.85);
-	qtableView.verticalHeader()->setFont(font);
+    // still needed on 2023.12.19
+    QFont font(qtableView.verticalHeader()->font());
+    auto sz(font.pointSizeF());
+    font.setPointSizeF(sz * 0.85);
+    qtableView.verticalHeader()->setFont(font);
+#endif
+}
+
+
+void setHeaderColor(QTableView* pTableView)
+{
+#ifdef WIN32   //ttt0: Review where is this actually needed. As of 2023.12.19, on Windows the header is white and the
+    // cells are also white.
+    pTableView->setStyleSheet("QHeaderView::section { background-color:#f8f8f8 }");
 #endif
 }
 
@@ -1873,3 +1773,26 @@ void decreaseRowHeaderFont(QTableView& qtableView)
 
 
 
+
+
+//=============================================================================================
+//=============================================================================================
+//=============================================================================================
+
+void setMp3DiagsUserAgent(QNetworkRequest& request)
+{
+    request.setRawHeader("User-Agent", "Mp3Diags/1.5 ( https://mp3diags.sourceforge.net/ )");
+    //header.setValue("User-Agent", "Firefox 52");
+}
+
+void setFirefoxUserAgent(QNetworkRequest& request)
+{
+    request.setRawHeader("User-Agent", "Mozilla/5.0 (X11; Linux x86_64; rv:120.0) Gecko/20100101 Firefox/120.0");
+}
+
+void setAcceptGzip(QNetworkRequest& request)
+{
+    request.setRawHeader("Accept-Encoding", "gzip");
+}
+
+
